@@ -11,10 +11,12 @@ from typing import Callable as _Callable
 from typing import Iterable as _Iterable
 from typing import List as _List
 from typing import MutableMapping as _MutableMapping
+from typing import NamedTuple
 from typing import Union as _Union
 
 # third-party
 from vcorelib.asyncio import run_handle_stop as _run_handle_stop
+from vcorelib.io.types import JsonObject as _JsonObject
 from vcorelib.logging import LoggerMixin as _LoggerMixin
 from vcorelib.logging import LoggerType as _LoggerType
 from vcorelib.namespace import Namespace as _Namespace
@@ -25,17 +27,25 @@ from runtimepy.net.connection import Connection as _Connection
 from runtimepy.net.manager import ConnectionManager as _ConnectionManager
 
 ConnectionMap = _MutableMapping[str, _Connection]
-NetworkApplication = _Callable[
-    [_AsyncExitStack, ConnectionMap], _Awaitable[int]
-]
+
+
+class AppInfo(NamedTuple):
+    """References provided to network applications."""
+
+    stack: _AsyncExitStack
+    connections: ConnectionMap
+    stop: _asyncio.Event
+    config: _JsonObject
+
+
+NetworkApplication = _Callable[[AppInfo], _Awaitable[int]]
 ServerTask = _Awaitable[None]
 
 
-async def init_only(stack: _AsyncExitStack, connections: ConnectionMap) -> int:
+async def init_only(app: AppInfo) -> int:
     """A network application that doesn't do anything."""
 
-    del stack
-    del connections
+    del app
     return 0
 
 
@@ -50,6 +60,8 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
         stop_sig: _asyncio.Event = None,
         namespace: _Namespace = None,
         logger: _LoggerType = None,
+        app: NetworkApplication = init_only,
+        config: _JsonObject = None,
     ) -> None:
         """Initialize this connection arbiter."""
 
@@ -65,6 +77,15 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
         _NamespaceMixin.__init__(self, namespace=namespace)
 
+        # A fallback application. Set a class attribute so this can be more
+        # easily externally updated.
+        self._app = app
+
+        # Application configuration data.
+        if config is None:
+            config = {}
+        self._config = config
+
         # Keep track of connection objects.
         self._connections: ConnectionMap = {}
         self._deferred_connections: _MutableMapping[
@@ -114,7 +135,10 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
         return result
 
     async def _entry(
-        self, app: NetworkApplication, check_connections: bool = True
+        self,
+        app: NetworkApplication = None,
+        check_connections: bool = True,
+        config: _JsonObject = None,
     ) -> int:
         """
         Ensures connections are given a chance to initialize, run the
@@ -149,7 +173,18 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
             ):
                 async with _AsyncExitStack() as stack:
                     self.logger.info("Application starting.")
-                    result = await app(stack, self._connections)
+
+                    if app is None:
+                        app = self._app
+
+                    result = await app(
+                        AppInfo(
+                            stack,
+                            self._connections,
+                            self.stop_sig,
+                            config if config is not None else self._config,
+                        )
+                    )
                     self.logger.info("Application returned %d.", result)
 
         finally:
@@ -161,15 +196,18 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
     async def app(
         self,
-        app: NetworkApplication = init_only,
+        app: NetworkApplication = None,
         check_connections: bool = True,
+        config: _JsonObject = None,
     ) -> int:
         """
         Run the application alongside the connection manager and server tasks.
         """
 
         result = await _asyncio.gather(
-            self._entry(app, check_connections=check_connections),
+            self._entry(
+                app, check_connections=check_connections, config=config
+            ),
             self.manager.manage(self.stop_sig),
             *self._servers,
         )
@@ -177,16 +215,19 @@ class BaseConnectionArbiter(_NamespaceMixin, _LoggerMixin):
 
     def run(
         self,
-        app: NetworkApplication = init_only,
+        app: NetworkApplication = None,
         eloop: _asyncio.AbstractEventLoop = None,
         signals: _Iterable[int] = None,
         check_connections: bool = True,
+        config: _JsonObject = None,
     ) -> int:
         """Run the application until the stop signal is set."""
 
         return _run_handle_stop(
             self.stop_sig,
-            self.app(app=app, check_connections=check_connections),
+            self.app(
+                app=app, check_connections=check_connections, config=config
+            ),
             eloop=eloop,
             signals=signals,
         )
