@@ -9,7 +9,7 @@ from abc import abstractmethod as _abstractmethod
 import asyncio as _asyncio
 from asyncio import DatagramProtocol as _DatagramProtocol
 from asyncio import DatagramTransport as _DatagramTransport
-from logging import getLogger
+from logging import getLogger as _getLogger
 import socket as _socket
 from typing import Tuple as _Tuple
 from typing import Type as _Type
@@ -23,8 +23,11 @@ from vcorelib.logging import LoggerType as _LoggerType
 from runtimepy.net import IpHost, get_free_socket
 from runtimepy.net.connection import BinaryMessage as _BinaryMessage
 from runtimepy.net.connection import Connection as _Connection
+from runtimepy.net.connection import EchoConnection as _EchoConnection
 from runtimepy.net.mixin import TransportMixin as _TransportMixin
 
+LOG = _getLogger(__name__)
+
 
 class UdpQueueProtocol(_DatagramProtocol):
     """A simple UDP protocol that populates a message queue."""
@@ -67,7 +70,7 @@ class UdpConnection(_Connection, _TransportMixin):
         self._transport: _DatagramTransport = transport
 
         self._protocol = protocol
-        super().__init__(getLogger(self.logger_name("UDP ")))
+        super().__init__(_getLogger(self.logger_name("UDP ")))
         self._protocol.logger = self.logger
 
     @_abstractmethod
@@ -96,6 +99,8 @@ class UdpConnection(_Connection, _TransportMixin):
 
         eloop = _asyncio.get_event_loop()
 
+        LOG.debug("kwargs: %s", {**kwargs})
+
         transport: _DatagramTransport
         (
             transport,
@@ -142,3 +147,15 @@ class UdpConnection(_Connection, _TransportMixin):
             # If we failed to read a message, disable.
             if not result:
                 self.disable("read processing error")
+
+
+class EchoUdpConnection(UdpConnection, _EchoConnection):
+    """An echo connection for UDP."""
+
+    async def process_datagram(
+        self, data: bytes, addr: _Tuple[str, int]
+    ) -> bool:
+        """Process a datagram."""
+
+        self.sendto(data, addr=addr)
+        return True
