@@ -32,6 +32,7 @@ from websockets.server import serve as _serve
 # internal
 from runtimepy.net import sockname as _sockname
 from runtimepy.net.connection import BinaryMessage, Connection
+from runtimepy.net.connection import EchoConnection as _EchoConnection
 from runtimepy.net.manager import ConnectionManager as _ConnectionManager
 
 T = _TypeVar("T", bound="WebsocketConnection")
@@ -110,7 +111,27 @@ class WebsocketConnection(Connection):
             conn = cls(protocol)
             if init is None or await init(conn):
                 if manager is not None:
+                    # Allow the connection manager to process this connection.
                     await manager.queue.put(conn)
+
+                    # Wait for either the connection to be disabled, or for
+                    # the stop signal to be set.
+                    tasks = [_asyncio.create_task(conn.disabled_event.wait())]
+                    if stop_sig is not None:
+                        tasks.append(_asyncio.create_task(stop_sig.wait()))
+
+                    # Wait for the event and cancel the task that didn't
+                    # complete.
+                    _, pending = await _asyncio.wait(
+                        tasks,
+                        return_when=_asyncio.FIRST_COMPLETED,
+                    )
+                    for task in pending:
+                        task.cancel()
+                        await task
+
+                # If there's no connection manager, just process the
+                # connection here.
                 else:
                     await conn.process(stop_sig=stop_sig)
 
@@ -186,3 +207,7 @@ class WebsocketConnection(Connection):
             LOG.info("WebSocket Application stopped.")
 
         LOG.info("WebSocket Server closed.")
+
+
+class EchoWebsocketConnection(WebsocketConnection, _EchoConnection):
+    """An echo connection for WebSocket."""
